#!/usr/bin/env python3
# Copyright (c) 2022 by Silvan Calarco <silvan.calarco@mambasoft.it>
# Release under the terms of the GPL version 3 license

from PyQt5 import QtGui, QtCore, QtWidgets, uic
from PyQt5.QtCore import pyqtProperty
from PyQt5.QtCore import QLocale, QTranslator
from pathlib import Path
import gettext
import subprocess
import os
import distro

import gi
gi.require_version('PackageKitGlib', '1.0')
from gi.repository import PackageKitGlib as packagekit

DATADIR = os.path.dirname(os.path.realpath((__file__))) + "/"


class MambareportWizard(QtWidgets.QWizard):
    def __init__(self, parent=None):
        super(MambareportWizard, self).__init__(parent)
        self.setWindowTitle(_("Diagnostic report tool") +
            " - {}".format(distro.name()))
        self.setFixedSize(571,465)

        self.welcomePage = WelcomePage(self)
        self.welcomePage.distroInfoLabel.setText(distro.name(pretty=True) +
          "<br/><br/><a href=\""+distro.os_release_info()['home_url']+
          "\">" + distro.os_release_info()['home_url'] + "</a>")
        self.addPage(self.welcomePage)

        self.generateReportPage = GenerateReportPage(self)
        self.addPage(self.generateReportPage)
        self.currentIdChanged.connect(self.currentIdChangedSlot)
        self.generateReportThread = GenerateReportThread(self)

    def currentIdChangedSlot(self, currentId):
        if currentId == 1:
            self.generateReportThread.start()


class GenerateReportThread(QtCore.QThread):
    parent = None
    doneSignal = QtCore.pyqtSignal()
    updateProgressSignal = QtCore.pyqtSignal(dict)

    def __init__(self, parent=None):
        super(GenerateReportThread, self).__init__(parent)
        self.parent = parent
        self.doneSignal.connect(parent.generateReportPage.completeChanged)
        self.updateProgressSignal.connect(
            parent.generateReportPage.updateProgressSlot)

    def run(self):
        # Update packages list
        reportFile = os.path.expanduser("~") + '/openmamba-report.txt'
        self.updateProgressSignal.emit({ 'value': 5,
            'label': _("Generating report...")})
        result = subprocess.run(['pkexec', 'openmamba-makereport',
                                 '\'' + self.parent.welcomePage.reasonLineEdit.displayText() + '\'',
                                 reportFile ],
            stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        if result.returncode == 0:
            if result.stdout:
                self.updateProgressSignal.emit({
                    'details': result.stdout.decode('UTF-8').replace('\n', '<br/>')})
        else:
            if result.stderr:
                self.updateProgressSignal.emit({
                    'details': result.stderr.decode('UTF-8').replace('\n', '<br/>')})
        self.updateProgressSignal.emit({ 'value': 100,
            'label': _("Done. Report saved as <b>{}</b>.").format(reportFile)})
        # Enable back and next buttons
        self.parent.generateReportPage.done = True
        self.doneSignal.emit()


class WelcomePage(QtWidgets.QWizardPage):
    done = False

    def __init__(self, parent=None):
        super(WelcomePage, self).__init__(parent)
        uic.loadUi(DATADIR + "WelcomePage.ui", self)
        self.registerField("reasons*", self.reasonLineEdit)
        self.show()


class GenerateReportPage(QtWidgets.QWizardPage):
    done = False

    def __init__(self, parent=None):
        super(GenerateReportPage, self).__init__(parent)
        uic.loadUi(DATADIR + "GenerateReportPage.ui", self)
        self.show()

    def isComplete(self):
        return self.done

    @QtCore.pyqtSlot(dict)
    def updateProgressSlot(self, dict):
        if 'value' in dict:
            self.progressBar.setValue(dict['value'])
        if 'label' in dict:
            self.progressLabel.setText(dict['label'])
        if 'details' in dict:
            self.reportOutput.append("<br/>" + dict['details'])


if __name__ == '__main__':
    import sys
    app = QtWidgets.QApplication(sys.argv)
    app.setWindowIcon(QtGui.QIcon.fromTheme("mambareport"))
    # UI translations
    locale = QLocale.system().name()
    language = locale[0:2]
    qtTranslator = QTranslator()
    if qtTranslator.load("mambareport", "/usr/share/locale/" + locale + "/LC_MESSAGES/"):
        app.installTranslator(qtTranslator)
    elif qtTranslator.load("mambareport", "/usr/share/locale/" + language + "/LC_MESSAGES/"):
        app.installTranslator(qtTranslator)
    # PY translations
    gettext.install('mambareport', '/usr/share/locale')
    wizard = MambareportWizard()
    wizard.show()
    sys.exit(app.exec_())
